
# q-expansion of newform 2304.2.a.d, downloaded from the LMFDB on 07 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2304
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [-2], [0], [0], [4], [-8], [0], [0], [10], [0], [12], [8], [0], [0], [-14], [0], [12], [0], [0], [6], [0], [0], [16], [18], [2], [0], [0], [20], [16], [0], [0], [-8], [0], [14], [0], [12], [0], [0], [-26], [0], [20], [0], [14], [2], [0], [0], [0], [0], [4], [-16], [0], [-30], [0], [32], [0], [26], [0], [-28], [-32], [0], [34], [0], [0], [-26], [-22], [0], [-18], [0], [-36], [16], [0], [0], [-36], [0], [0], [-34], [12], [40], [-6], [0], [-28], [0], [-34], [0], [0], [-40], [42], [38], [0], [0], [0], [0], [0], [0], [0], [10], [40], [0], [20], [0], [38], [0], [40], [0], [2], [0], [-16], [0], [10], [0], [-36], [-32], [0], [0], [-8], [0], [0], [-26], [0], [12], [46], [-2], [0], [0], [10], [-44], [0], [0], [4], [0], [0], [0], [52], [-40], [-50], [34], [0], [22], [56], [0], [50], [0], [0], [20], [0], [-36], [8], [0], [0], [12], [32], [0], [0], [0], [0], [0], [40], [-38], [-58], [0], [-56], [0], [0], [8], [0], [0], [12], [-30], [46], [0], [60], [0], [6], [0], [64], [0], [62], [0], [60], [0], [0], [4], [-32], [0], [-50], [52], [0], [-54], [0], [-66], [0], [0], [-10], [0], [-64], [50], [-44], [-32], [0], [70], [0], [68], [30], [0], [-22], [0], [0], [-16], [0], [-2], [-50], [0], [0], [0], [-10], [0], [-40], [0], [-74], [68], [0], [-56], [0], [0], [60], [-16], [0], [0], [0], [-76], [0], [0], [32], [0], [0], [66], [14], [0], [0], [0], [0], [0], [-36], [-64], [0], [0], [0], [0], [0], [68], [80], [0], [6], [26], [0], [20], [0], [-62], [38], [0], [0], [-76], [-36], [8], [0], [70], [80], [0], [-34], [60], [0], [54], [0], [78], [0], [0], [-84], [-70], [0], [0], [0], [0], [60], [0], [0], [66], [-82], [0], [-80], [70], [0], [16], [0], [-84], [86], [0], [-46], [0], [0], [86], [-58], [0], [0], [0], [18], [0], [4], [0], [-84], [0], [50], [-40], [0], [0], [-90], [0], [0], [-66], [80], [0], [-58], [10], [0], [-56], [-30], [0], [0], [0], [94], [-28], [22], [0], [0], [0], [0], [60], [-16], [-90], [0], [-84], [-88], [-94], [0], [-86], [0], [-92], [0], [0], [82], [0], [42], [-70], [0], [-84], [-64], [0], [0], [-8], [0], [12], [-80], [0], [0], [0], [26], [-38], [0], [70], [0], [0], [0], [-14], [0], [-92], [0], [0], [34], [40], [-102], [22], [56], [0], [32], [0], [0], [0], [68], [0], [0], [-66], [94], [0], [0], [0], [-6], [0], [104], [0], [-50], [60], [104], [0], [-88], [34], [0], [-28], [40], [0], [0], [-46], [-82], [0], [0], [102], [-38], [0], [0], [-88], [0], [-106], [108], [0], [0], [-106], [58], [0], [80], [0], [0]]
